/**
 * Copyright (c) 2011-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.component.framework.junit.impl.mock;

import static org.mockito.Matchers.anyString;
import static org.mockito.Mockito.when;

import java.util.MissingResourceException;
import java.util.Vector;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.management.MBeanNames;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.xml.namespace.QName;

import org.mockito.Mockito;
import org.mockito.invocation.InvocationOnMock;
import org.mockito.stubbing.Answer;
import org.ow2.petals.component.framework.AbstractComponent;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import com.ebmwebsourcing.easycommons.lang.UncheckedException;

/**
 * @author Adrien Ruffie - EBM WebSourcing
 */
public class MockComponentContext implements ComponentContext {

    private final String installRoot;

    private final String workspaceRoot;

    private final Logger logger;

    private final MockDeliveryChannel deliveryChannel;

    private final MBeanServer mockMBeanServer;

    private final MBeanNames mockMBeanNames;

    private final String componentName;

    private static Vector<MockServiceEndpoint> serviceEndpoints = new Vector<MockServiceEndpoint>();

    private final AbstractComponent jbiComponent;

    public static void activateEndpoint(final MockServiceEndpoint serviceEndpoint) {
        MockComponentContext.serviceEndpoints.add(serviceEndpoint);
    }

    public MockComponentContext(final String componentName, final AbstractComponent jbiComponent,
            final String installRoot, final String workspaceRoot, final Logger logger) {
        this.componentName = componentName;
        this.installRoot = installRoot;
        this.workspaceRoot = workspaceRoot;
        this.jbiComponent = jbiComponent;
        this.logger = logger;
        this.deliveryChannel = new MockDeliveryChannel(logger);
        this.mockMBeanServer = Mockito.mock(MBeanServer.class);
        this.mockMBeanNames = Mockito.mock(MBeanNames.class);

        when(this.mockMBeanNames.createCustomComponentMBeanName(anyString())).thenAnswer(new Answer<ObjectName>() {
            public ObjectName answer(final InvocationOnMock invocation) {
                final Object[] args = invocation.getArguments();
                try {
                    return new ObjectName("Petals-CDK-Unit-Test:type=custom,name=" + args[0].toString());
                } catch (final MalformedObjectNameException e) {
                    throw new UncheckedException(e);
                } catch (final NullPointerException e) {
                    throw new UncheckedException(e);
                }
            }
        });

    }

    @Override
    public ServiceEndpoint activateEndpoint(final QName serviceName, final String endpointName)
            throws JBIException {
        final MockServiceEndpoint mockServiceEndpoint = new MockServiceEndpoint(endpointName, serviceName);
        final Document description = this.jbiComponent.getServiceDescription(mockServiceEndpoint);
        mockServiceEndpoint.setDescription(description);
        MockComponentContext.activateEndpoint(mockServiceEndpoint);
        return mockServiceEndpoint;
    }

    @Override
    public void deactivateEndpoint(final ServiceEndpoint endpoint) throws JBIException {
        MockComponentContext.serviceEndpoints.remove(endpoint);
    }

    @Override
    public void deregisterExternalEndpoint(final ServiceEndpoint externalEndpoint) throws JBIException {
        // TODO Auto-generated method stub
    }

    @Override
    public String getComponentName() {
        return this.componentName;
    }

    @Override
    public MockDeliveryChannel getDeliveryChannel() {
        return this.deliveryChannel;
    }

    @Override
    public ServiceEndpoint getEndpoint(final QName service, final String name) {
        return this.resolveServiceEndpoint(service, name);
    }

    private MockServiceEndpoint resolveServiceEndpoint(final QName service, final String name) {
        MockServiceEndpoint result = null;
        for (final MockServiceEndpoint serviceEndpoint : MockComponentContext.serviceEndpoints) {
            if (name.equals(serviceEndpoint.getEndpointName())
                    && service.equals(serviceEndpoint.getServiceName())) {
                result = serviceEndpoint;
                break;
            }
        }

        return result;
    }

    public static ServiceEndpoint resolveServiceEndpoint(final QName service) {
        ServiceEndpoint result = null;

        // try to resolve with service name only, get the first ep for this
        // service
        for (final ServiceEndpoint serviceEndpoint : MockComponentContext.serviceEndpoints) {
            if (service.equals(serviceEndpoint.getServiceName())) {
                result = serviceEndpoint;
                break;
            }
        }

        return result;
    }

    @Override
    public Document getEndpointDescriptor(final ServiceEndpoint endpoint) throws JBIException {
        final MockServiceEndpoint mockServiceEnpoint = this.resolveServiceEndpoint(endpoint.getServiceName(),
                endpoint.getEndpointName());
        final Document description = mockServiceEnpoint.getDescription();
        return description;
    }

    @Override
    public ServiceEndpoint[] getEndpoints(final QName interfaceName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public ServiceEndpoint[] getEndpointsForService(final QName serviceName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public ServiceEndpoint[] getExternalEndpoints(final QName interfaceName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public ServiceEndpoint[] getExternalEndpointsForService(final QName serviceName) {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String getInstallRoot() {
        return this.installRoot;
    }

    @Override
    public Logger getLogger(final String suffix, final String resourceBundleName)
            throws MissingResourceException, JBIException {
        return this.logger;
    }

    @Override
    public MBeanNames getMBeanNames() {
        return this.mockMBeanNames;
    }

    @Override
    public MBeanServer getMBeanServer() {
        return this.mockMBeanServer;
    }

    @Override
    public InitialContext getNamingContext() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public Object getTransactionManager() {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String getWorkspaceRoot() {
        return workspaceRoot;
    }

    @Override
    public void registerExternalEndpoint(final ServiceEndpoint externalEndpoint) throws JBIException {
        // TODO Auto-generated method stub
    }

    @Override
    public ServiceEndpoint resolveEndpointReference(final DocumentFragment epr) {
        // TODO Auto-generated method stub
        return null;
    }
}
